==> know_its_ok-0.1.0.tar.gz: know_its_ok-0.1.0/know_its_ok.py <==
#!/usr/bin/env python3

import sys, os, re, subprocess, pytest

__version__ = '0.1.0'
test_pattern = re.compile(r'\d{2}_test_.+\.py')

def main():
    """
    Discover and run the tests for this package.
    """
    cli_args = sys.argv[1:]
    repo_dir = discover_repository()
    tests = discover_tests(repo_dir)
    cli_args, tests = pick_tests(cli_args, tests)
    success = run_pytest(repo_dir, cli_args, tests)
    sys.exit(success)

def discover_repository():
    """
    Return the path the root of the git repository containing the current 
    working directory.

    This is important because everything else in this script will orient itself 
    relative to this directory.  If the git repository can't be found for any 
    reason (e.g. the current working directory isn't part of a repository, git 
    isn't installed, etc.) a ValueError will be raised.
    """
    try: 
        with open(os.devnull, 'w') as devnull:
            command = 'git', 'rev-parse', '--show-toplevel'
            stdout = subprocess.check_output(command, stderr=devnull)
            path = stdout.strip()

    except subprocess.CalledProcessError:
        raise ValueError("couldn't find git repository")

    return path.decode('utf8')

def discover_tests(repo_dir):
    """
    Return the names of all the test scripts.

    Test scripts must be located in ``<repo>/tests`` and must follow the 
    pattern defined by ``test_pattern``.  By default, this pattern is roughly 
    the glob ``??_test_*.py``.  If the tests directory doesn't exist in the 
    expected location, an exception will be raised.
    """
    test_dir = os.path.join(repo_dir, 'tests')
    if not os.path.isdir(test_dir):
        raise ValueError("couldn't find tests/ directory")
    return [os.path.basename(x)
            for x in os.listdir(test_dir)
            if test_pattern.match(x)]

def pick_tests(cli_args, tests):
    """
    Return the tests the user actually wants to run, considering the given 
    command line arguments.

    Each discovered test is compared to each argument the user specified.  If 
    any argument is a substring of a test name, that test will be included in 
    the returned list.  For example, if one of the arguments is 'foo' and a 
    test named 'test_foo.py' exists, that test will be included.  If no tests 
    are matched, the complete list of tests is returned.  So unless the user 
    specifies otherwise, all the tests will be run.  Also returned is a list of 
    all the arguments that didn't match the name of a test.
    """
    pytest_args = cli_args[:]
    matched_tests = []

    for test in tests:
        for arg in cli_args:
            if arg in test:
                matched_tests.append(test)
                cli_args.remove(arg)
            
    return pytest_args, matched_tests or tests
    
def sort_tests(tests):
    """
    Determine the order in which the tests should be run.

    By default, all the tests have a numeric prefix which indicates the order 
    they're meant to be run in, so we can simply sort the tests numerically.
    """
    return sorted(tests)

def run_pytest(repo_dir, cli_args, tests):
    """
    Run pytest in the right directory.  
    
    Use the arguments specified on the command line on top of my preferred set 
    of default arguments.  My default arguments stop on the first failure, 
    collect coverage data, and turn on color output.
    """
    args = [
        '-x',
        '--color', 'yes',
        '--cov', deduce_package_name(repo_dir),
        '--cov-report', 'html',
    ]
    args += cli_args
    args += sort_tests(tests)

    from signal import signal, SIGPIPE, SIG_DFL
    signal(SIGPIPE, SIG_DFL)

    test_dir = os.path.join(repo_dir, 'tests')
    os.chdir(test_dir)
    return pytest.main(args)

def deduce_package_name(repo_dir):
    """
    Figure out what the name of the package being tested.

    This information is needed by coverage.  It is deduced by walking up the 
    directory tree looking for the first directory with an ``__init__.py`` 
    file.  When such a directory is found, its name is immediately returned.  
    If no suitable directory  is found, a ValueError is raised.
    """
    for root, dirs, files in os.walk(repo_dir):
        if '__init__.py' in files:
            return os.path.basename(root)
    raise ValueError("couldn't find ``__init__.py``")

==> know_its_ok-0.2.0.tar.gz: know_its_ok-0.2.0/know_its_ok.py <==
#!/usr/bin/env python3

import sys, os, re, subprocess, pytest

__version__ = '0.2.0'
test_pattern = re.compile(r'\d{2}_test_.+\.py')

def main():
    """
    Discover and run the tests for this package.
    """
    cli_args = sys.argv[1:]
    repo_dir = discover_repository()
    test_dir, tests = discover_tests(repo_dir)
    cli_args, tests = pick_tests(cli_args, tests)
    success = run_pytest(repo_dir, test_dir, cli_args, tests)
    sys.exit(success)

def discover_repository():
    """
    Return the path the root of the git repository containing the current 
    working directory.

    This is important because everything else in this script will orient itself 
    relative to this directory.  If the git repository can't be found for any 
    reason (e.g. the current working directory isn't part of a repository, git 
    isn't installed, etc.) a ValueError will be raised.
    """
    try: 
        with open(os.devnull, 'w') as devnull:
            command = 'git', 'rev-parse', '--show-toplevel'
            stdout = subprocess.check_output(command, stderr=devnull)
            path = stdout.strip()

    except subprocess.CalledProcessError:
        raise ValueError("couldn't find git repository")

    return path.decode('utf8')

def discover_tests(repo_dir):
    """
    Return the names of all the test scripts.

    Test scripts must be located in ``<repo>/tests`` and must follow the 
    pattern defined by ``test_pattern``.  By default, this pattern is roughly 
    the glob ``??_test_*.py``.  If the tests directory doesn't exist in the 
    expected location, an exception will be raised.
    """
    test_dir = os.path.join(repo_dir, 'tests')
    if not os.path.isdir(test_dir):
        raise ValueError("couldn't find tests/ directory")
    return test_dir, [
            os.path.basename(x)
            for x in os.listdir(test_dir)
            if test_pattern.match(x)]

def pick_tests(cli_args, tests):
    """
    Return the tests the user actually wants to run, considering the given 
    command line arguments.

    Each discovered test is compared to each argument the user specified.  If 
    any argument is a substring of a test name, that test will be included in 
    the returned list.  For example, if one of the arguments is 'foo' and a 
    test named 'test_foo.py' exists, that test will be included.  If no tests 
    are matched, the complete list of tests is returned.  So unless the user 
    specifies otherwise, all the tests will be run.  Also returned is a list of 
    all the arguments that didn't match the name of a test.
    """
    pytest_args = cli_args[:]
    matched_tests = []

    for test in tests:
        for arg in cli_args:
            if arg in test:
                matched_tests.append(test)
                cli_args.remove(arg)
            
    return pytest_args, matched_tests or tests
    
def sort_tests(tests):
    """
    Determine the order in which the tests should be run.

    By default, all the tests have a numeric prefix which indicates the order 
    they're meant to be run in, so we can simply sort the tests numerically.
    """
    return sorted(tests)

def run_pytest(repo_dir, test_dir, cli_args, tests):
    """
    Run pytest in the right directory.  
    
    Use the arguments specified on the command line on top of my preferred set 
    of default arguments.  My default arguments stop on the first failure, 
    collect coverage data, and turn on color output.
    """
    args = [
        '-x',
        '--color', 'yes',
        '--cov', deduce_package_name(repo_dir),
        '--cov-report', 'html',
    ]
    args += cli_args
    args += sort_tests(tests)

    from signal import signal, SIGPIPE, SIG_DFL
    signal(SIGPIPE, SIG_DFL)

    os.chdir(test_dir)
    return pytest.main(args)

def deduce_package_name(repo_dir):
    """
    Figure out what the name of the package being tested.

    This information is needed by coverage.  It is deduced by walking up the 
    directory tree looking for the first directory with an ``__init__.py`` 
    file.  When such a directory is found, its name is immediately returned.  
    If no suitable directory  is found, a ValueError is raised.
    """
    for root, dirs, files in os.walk(repo_dir):
        if '__init__.py' in files:
            return os.path.basename(root)
    raise ValueError("couldn't find ``__init__.py``")
